# style and layout settings for PDF
PAGE_ORIENTATION_DEFAULT = "portrait"
PAGE_FORMAT_DEFAULT = "a4"
PAGE_UNITS_DEFAULT = "mm"
FONT_FAMILY_DEFAULT = "NotoSans"
FONT_FAMILY_MONO_DEFAULT = "NotoSansMono"
FONT_SIZE_NORMAL = 12
FONT_SIZE_LARGE = 14
FONT_SIZE_SMALL = 10
LINE_HEIGHT_DEFAULT = 6
LINE_HEIGHT_SMALL = 2
MARGIN_LEFT = 10
TAB_WIDTH = 4

# slack
MINUTES_UNTIL_USERNAME_REPEATS = 10
MAX_MESSAGES_PER_CHANNEL = 10000
SLACK_PAGE_LIMIT = 200
